@@ -17,7 +17,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 #ifndef DBUS_MESSAGE_INTERNAL_H
@@ -37,8 +37,6 @@ void _dbus_message_get_network_data  (DBusMessage       *message,
 
 void        _dbus_message_lock                  (DBusMessage  *message);
 void        _dbus_message_unlock                (DBusMessage  *message);
-void        _dbus_message_set_serial            (DBusMessage  *message,
-                                                 dbus_uint32_t serial);
 dbus_bool_t _dbus_message_add_size_counter      (DBusMessage  *message,
                                                  DBusCounter  *counter);
 void        _dbus_message_add_size_counter_link (DBusMessage  *message,
